@@ -29,14 +29,14 @@ import yparse
 from ply import *
 
 if len(sys.argv) == 1:
-    print "usage : yply.py [-nocode] inputfile"
+    print("usage : yply.py [-nocode] inputfile")
     raise SystemExit
 
 if len(sys.argv) == 3:
     if sys.argv[1] == '-nocode':
         yparse.emit_code = 0
     else:
-        print "Unknown option '%s'" % sys.argv[1]
+        print("Unknown option '%s'" % sys.argv[1])
         raise SystemExit
     filename = sys.argv[2]
 else:
@@ -44,8 +44,8 @@ else:
 
 yacc.parse(open(filename).read())
 
-print """
+print("""
 if __name__ == '__main__':
     from ply import *
     yacc.yacc()
-"""
+""")
