@@ -14,7 +14,7 @@
 # such tokens
 # -----------------------------------------------------------------------------
 import sys
-sys.path.insert(0,"..")
+if ".." not in sys.path: sys.path.insert(0,"..")
 
 import ply.lex as lex
 
@@ -36,7 +36,7 @@ def t_H_EDIT_DESCRIPTOR(t):
     return t                                  
     
 def t_error(t):
-    print "Illegal character '%s'" % t.value[0]
+    print("Illegal character '%s'" % t.value[0])
     t.lexer.skip(1)
     
 # Build the lexer
