@@ -1,9 +1,9 @@
-# lex_token.py
+# lex_token2.py
 #
 # Tests for tokens of wrong type
 
 import sys
-sys.path.insert(0,"..")
+if ".." not in sys.path: sys.path.insert(0,"..")
 
 import ply.lex as lex
 
@@ -16,7 +16,6 @@ t_NUMBER = r'\d+'
 def t_error(t):
     pass
 
-sys.tracebacklimit = 0
 
 lex.lex()
 
